
function [U,Y] = dataDyn(n,nu,m,k,sigma)

%   [U,Y] = dataDyn(n,nu,m,k,sigma)
%   [U,Y] = dataDyn
%
% Generate random data coming from a state-space system.
%
% Input parameters:
%  - n: State dimension (default 1)
%  - nu: Input dimension (default 1)
%       can be zero, meaning stochastic process,
%       or given predetermined input sequence
%  - m: Output dimension (default 1)
%  - k: Number of data samples (default 100)
%  - sigma: Standard deviation of the noise (default 0) 
% Return parameters:
%  - U: Input sequence
%  - Y: Output sequence
%
% Heikki Hyotyniemi Feb.20, 2001


if nargin < 1
   n = 1;
end
if nargin < 2
   nu = 1;
end
if nargin < 3
   m = 1;
end
if nargin < 4
   k = 100;
end
if nargin < 5
   sigma = 0;
end
if nu==0
   nu = 1;
   U = zeros(k,nu);
elseif length(nu)>1
   U = nu;
   [k,nu] = size(U);
else
   U = randn(k,nu);
end

A = randn(n,n);
A = A/max(abs(eig(A)))/1.1  % Stabilize!
B = randn(n,nu)
C = randn(m,n)
D = randn(m,nu)

eig(A)

E = sigma*randn(k,n+m);
EE = E*randn(n+m,n+m);
E1 = EE(:,1:n);
E2 = EE(:,n+1:n+m);
X = zeros(k+1,n);
Y = zeros(k,m);

for i = 1:k
   Y(i,:) = (C*X(i,:)')' + (D*U(i,:)')' + E2(i,:);
   X(i+1,:) = (A*X(i,:)')' + (B*U(i,:)')' + E1(i,:);
end

