
function [Exu,Uhat,Q] = regrFACTOR(U,Exu,Q,lambda,nonlin,XXref)

%   [Exu,Uhat,Q] = regrFACTOR(U,Exu,Q,lambda,nonlin,XXref)
%
% Function fdor feedback adaptation in the "neocybernetic model".
%
% Inputs:
%  - U       Input data (format k x m)
%  - Exu     Model matrix (format n x m)
%  - Q       "Stiffnesses" (diagonal matrix n x n)
%  - lambda  Forgetting factor (scalar, default no learning)
%  - nonlin  Nonlinearity applied (default '1'=true)
%  - XXref   scalar/vector of x variances (default levels at 1)
% Outputs:
%  - Exu     Modified model matrix
%  - Uhat    Balance data estimate outside the system
%  - Q       Related to error covariances (diagonal matrix n x n)
%
% Heikki Hytyniemi  March 1, 2006


Ubar = NaN; Xbar = NaN; Uhat = NaN;
[k,m] = size(U);
[n,m1] = size(Exu);

if m~=m1, disp('Matrix U incompatible with the model'); return; end

if nargin<4, lambda = 1; end                % Default: No learning
if nargin<5, nonlin = 1; end                % Default: Better convergence
if nargin<6, XXref = 1; end                 % Default: All identical
if length(XXref)==1, XXref = XXref*ones(n,1); end


Xbar = U * (inv(eye(n)+Q*Exu*Exu'*Q')*Q*Exu)';

if nonlin == 1
  Xbar = Xbar.*(Xbar>0);
end
Exx = Xbar'*Xbar/k;

Uhat = Xbar * Q*Exu;
Ubar = U - Uhat;

Exu = lambda*Exu + (1-lambda)*Xbar'*Ubar/k; 

Q = Q.^lambda * diag(exp(XXref-diag(Exx)).^(1-lambda)); 
% Error covariance estimate Q^(-1/2)
