
function [f] = regrIdent(u,y,N,lambda,f)

%   [f] = regrIdent(u,y,N,lambda,f)
%   [f] = regrIdent(u,y,N,lambda)
%   [f] = regrIdent(u,y,N)
%
% Identify SISO system recursively.
%
% Input parameters:
%  - u: Scalar input sequence
%  - y: Scalar output sequence
%  - N: System dimension
%  - lambda: Forgetting factor (default 1, no forgetting)
%  - f: Initial parameter vector (default zero)
% Return parameters:
%  - f: Final parameter vector
%
% Heikki Hyotyniemi Feb.20, 2001


f = NaN;
[ku,n] = size(u);
[ky,m] = size(y);
if ku~=ky, disp('Input and output sequences incompatible!'); return;
else k = ku; end
if n~=1 | m~=1, disp('SISO system assumed!'); return; end

if nargin < 4
   lambda = 1;
end
if nargin < 5
   f = zeros(2*N,1);
end

X = zeros(k-N,2*N);
Y = y(N+1:k);
F = zeros(k-N,2*N);
for i = 1:N
   X(:,i) = u(i:k-N+i-1);
end
for i = 1:N
   X(:,N+i) = y(i:k-N+i-1);
end

R = 0.01*eye(2*N);
for i = 1:length(Y)
   F(i,:) = f';
   R = lambda*R + X(i,:)'*X(i,:);
   f = f + inv(R)*X(i,:)'*(Y(i)-X(i,:)*f);
end

figure; clf
plot(F); 
hold on;
fMLR = regrMLR(X,Y);
plot(ones(k-N,1)*fMLR');
