
function [A,B,C,D,Rxx,Ryy,Rxy] = regrSSI(U,Y,maxdim,N)

%   [A,B,C,D,Rxx,Ryy,Rxy] = regrSSI(U,Y,maxdim,N)
%   [A,B,C,D,Rxx,Ryy,Rxy] = regrSSI(U,Y,maxdim)
%
% Subspace identification (simplified and biased)
%
% Input parameters:
%  - U: Input data block (k x nu)
%  - Y: Output data block (k x m)
%  - maxdim: Assumed maximum possible system dimension
%  - N: System dimension (optional)
% Return parameters:
%  - A,B,C,D: System matrices
%  - Rxx,Ryy,Rxy: Noise covariances
%
% Heikki Hyotyniemi Sep.12, 2000


A = NaN; B = NaN; C = NaN; D = NaN;
Rxx = NaN; Ryy = NaN; Rxy = NaN; 
[ku,nu] = size(U);
[ky,m] = size(Y);
if ku == ky
   k = ku;
else disp('Input and output dimensions incompatible'); return;
end

UU = zeros(k-2*maxdim+1,2*maxdim*nu);
for i = 1:2*maxdim
   UU(:,(i-1)*nu+1:i*nu) = U(i:k-2*maxdim+i,:);
end
UP = UU(:,1:maxdim*nu);
UF = UU(:,maxdim*nu+1:2*maxdim*nu);

YY = zeros(k-2*maxdim+1,2*maxdim*m);
for i = 1:2*maxdim
   YY(:,(i-1)*m+1:i*m) = Y(i:k-2*maxdim+i,:);
end
YP = YY(:,1:maxdim*m);
YF = YY(:,maxdim*m+1:2*maxdim*m);

WP = [UP,YP];
WW = [WP,UF];
W0 = [WP,zeros(size(UF))];

OO = W0*pinv(WW'*WW)*WW'*YF;

if nargin==4
   theta = regrPCA(OO,N);
else
   theta = regrPCA(OO);
   N = size(theta,2);
end

X = OO*theta;
XP = X(1:size(X,1)-1,:);
XF = X(2:size(X,1),:);

LHS = [XF,YF(1:size(YF,1)-1,1:m)];
RHS = [XP,UF(1:size(UF,1)-1,1:nu)];
RR = RHS'*RHS;
ABCD = (pinv(RR)*RHS'*LHS)';

A = ABCD(1:N,1:N);
B = ABCD(1:N,N+1:N+nu);
C = ABCD(N+1:N+m,1:N);
D = ABCD(N+1:N+m,N+1:N+nu);

E = LHS - RHS*ABCD';
EE = E'*E/k;
Rxx = EE(1:N,1:N);
Ryy = EE(N+1:N+m,N+1:N+m);
Rxy = EE(1:N,N+1:N+m);

[Yhat0,Yhat1] = regrKalman(U,Y,A,B,C,D,Rxx,Ryy,Rxy);

fig = figure;
clf;
set(fig,'Name','Matching the model against data');
for i = 1:m
    subplot(m,1,i);
    hold on;
    plot(Y(:,i),'*');
    plot(Yhat0(:,i),'r');
    plot(Yhat1(:,i),'g');
end
legend('Data','Filter output','One-step predictor');
title('Estimate (solid lines) against the measurements');
hold off;
