
function [A,C,Rxx,Ryy,Rxy,X] = regrSSSI(Y,maxdim,N)

%   [A,C,Rxx,Ryy,Rxy] = regrSSSI(Y,maxdim,N)
%   [A,C,Rxx,Ryy,Rxy] = regrSSSI(Y,maxdim)
%
% Stochastic SubSpace Identification (simplified and biased)
%
% Input parameters:
%  - Y: Output data block (k x m)
%  - maxdim: Assumed maximum possible system dimension
%  - N: System dimension (optional)
% Return parameters:
%  - A,C: System matrices
%  - Rxx,Ryy,Rxy: Noise covariance matrices
%  - X: State sequence (k x N)
%
% Heikki Hyotyniemi Feb.1, 2001


A = NaN; C = NaN;
Rxx = NaN; Ryy = NaN; Rxy = NaN; X = NaN;
[k,m] = size(Y);

YY = zeros(k-2*maxdim+1,2*maxdim*m);
for i = 1:2*maxdim
   YY(:,(i-1)*m+1:i*m) = Y(i:k-2*maxdim+i,:);
end
YP = YY(:,1:maxdim*m);
YF = YY(:,maxdim*m+1:2*maxdim*m);

if nargin==3
   theta = regrPCA(YP,N);
else
   theta = regrPCA(YP);
   N = size(theta,2);
end

X = YP*theta;
XP = X(1:size(X,1)-1,:);
XF = X(2:size(X,1),:);

LHS = [XF,YP(1:size(YF,1)-1,(maxdim-1)*m+1:maxdim*m)];
RHS = [XP];
RR = RHS'*RHS;
AC = (pinv(RR)*RHS'*LHS)'; 

A = AC(1:N,1:N);
C = AC(N+1:N+m,1:N);

E = LHS - RHS*AC';
EE = E'*E/k;
Rxx = EE(1:N,1:N);
Ryy = EE(N+1:N+m,N+1:N+m);
Rxy = EE(1:N,N+1:N+m);

[Yhat0,Yhat1] = regrKalm(Y,A,C,Rxx,Ryy,Rxy);

fig = figure;
clf;
set(fig,'Name','Matching the model against data');
for i = 1:m
    subplot(m,1,i);
    hold on;
    plot(Y(:,i),'*');
    plot(Yhat0(:,i),'r');
    plot(Yhat1(:,i),'g');
end
legend('Data','Filter output','One-step predictor');
title('Estimate (solid lines) against the measurements');
hold off;

